/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ThumbsDown.
 *
 * The Initial Developer of the Original Code is
 * Atte Kemppilä.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */


function ModePage() // extend WizardPage
{
    var depends = "selected linkUrlTokens name";
    WizardPage.call(this, depends);
}

ModePage.prototype =
{
    get mode()      { return $("rgroup_mode"); },
    get modeValue() { return parseInt(this.mode.value); },
    
    get scriptId()      { return ((this.modeValue == MODE_MODIFY) ? parseInt($("mlist_mode_scripts").value) : null); },
    set scriptId(value) { $("mlist_mode_scripts").value = value; },
    
    get script() { return (this.scriptId ? scriptMgr.getScript(this.scriptId) : null); },
    
    onPageShow: function()
    {
        if (this.firstRun)
            $("col_mode_label").style.width = _("css.modeLabelColumn.width");
        
        if (this.dirty.selected)
        {
            this.mode.selectedIndex = -1;
            
            var rlbox = $("rlbox_mode");
            while (rlbox.itemCount > 0)
                rlbox.removeItemAt(0);
            for each (var thumbnail in model.selected)
            {
                var rlitem = rlbox.appendItem(thumbnail.anchor.link);
                rlitem.firstChild.crop = "none";
            }
        }
        
        if (this.dirty.linkUrlTokens || (this.dirty.name && (this.modeValue != MODE_MODIFY)))
        {
            if (! model.name)
                model.name = scriptWizard.getName(model.linkUrlTokens);
            
            // TODO: don't use sqlExec directly here
            var scripts = scriptMgr.sqlExec.fetchAll("getScriptsWithName", model.name);
            if (scripts.length == 1)
            {
                this.scriptId = scripts[0].id;
                if (this.modeValue != MODE_CREATE)
                    this.mode.value = MODE_MODIFY;
            }
        }
        
        if (this.modeValue != MODE_MODIFY)
            this.mode.value = MODE_CREATE;
        
        if (this.dirty.page)
            this._updateScript();
    },
    
    onModeSelect: function()
    {
        this._updateScript();
        wizard.updateWizardButtons();
    },
    
    onScriptSelect: function()
    {
        this._updateScript();
        wizard.updateWizardButtons();
    },
    
    canAdvance: function()
    {
        switch (this.modeValue)
        {
            case MODE_CREATE:
                return true;
                
            case MODE_MODIFY:
                if (this.script)
                    switch (this.script.type)
                    {
                        case TYPE_DIRECT:
                            return (model.target == TARGET_IMAGE);
                        case TYPE_REGEXP:
                        case TYPE_WRAPPED:
                            return (model.target == TARGET_PAGE);
                    }
                break;
        }
        return false;
    },
    
     _updateScript: function()
    {
        var script = this.script;
        var showAll = false;
        
        if (script)
        {
            $("lbl_mode_script_matcher").value = script.matcher;
            $("lbl_mode_script_type").value = _("type." + script.type);
            
            switch (script.type)
            {
                case TYPE_REGEXP:
                case TYPE_WRAPPED:
                    showAll = true;
                    $("lbl_mode_script_subType").previousSibling.value = (script.subType ? _("type." + script.type + ".subType") + ":" : "");
                    $("lbl_mode_script_subType").value = (script.subType ? _("type." + script.type + "." + script.subType) : "");
                    $("lbl_mode_script_action1").previousSibling.value = (script.action1 ? _("type." + script.type + ".action1") + ":" : "");
                    $("lbl_mode_script_action1").value = script.action1;
                    $("lbl_mode_script_action2").previousSibling.value = (script.action2 ? _("type." + script.type + ".action2") + ":" : "");
                    $("lbl_mode_script_action2").value = script.action2;
            }
        }
        
        $("mlist_mode_scripts").parentNode.parentNode.hidden = (this.modeValue != MODE_MODIFY);
        $("lbl_mode_script_matcher").parentNode.hidden       = (! script);
        $("lbl_mode_script_type").parentNode.hidden          = (! script);
        $("lbl_mode_script_subType").parentNode.hidden       = (! (script && script.subType && showAll));
        $("lbl_mode_script_action1").parentNode.hidden       = (! (script && script.action1 && showAll));
        $("lbl_mode_script_action2").parentNode.hidden       = (! (script && script.action2 && showAll));
    },
    
    onPageAdvanced: function()
    {
        model.mode = this.modeValue;
        
        switch (this.modeValue)
        {
            case MODE_CREATE:
                model.script = null;
                model.type = ((model.target == TARGET_IMAGE) ? TYPE_DIRECT : 0);
                break;
                
            case MODE_MODIFY:
                model.script = this.script;
                model.type = this.script.type;
                break;
        }
        
        return true;
    }
}

extend(ModePage, WizardPage);

pages.mode = new ModePage();
